#!-PERL-
# f2htranslate.pl
#
# THIS FILE IS COPYRIGHT 1994 BY TELENOR RESEARCH, NORWAY
#
#                    ALL RIGHTS RESERVED.
#
# For rights granted to the public see the COPYRIGHT file
#
# Programming : Jon Stephenson von Tetzchner

# This script is part of the fm2html package.

# Input:  <miffile> with ".mif" stripped, i.e. <basename>
# Output: <basename>.tmp.000H

$\ = "\n";		# set output record separator
$, = " ";		# set output field separator
$* = 0;			# set multiple line pattern matching

select( STDERR );	# all error messages go to stderr
$| = 1;			# make stderr unbuffered

$FALSE = 0;
$TRUE  = 1;


sub escape	# take string, insert backslash before special characters
{
  local( $_ ) = @_;
  s|([\.\+\?\*\'\"\(\)])|\\$1|g;
  $_;
}

sub basename	# take pathname, strip leading directories and optional suffix
{
  local( $_, $suffix ) = @_;
  m|/|o && s|^.*/([^/]*)$|$1|;
  $suffix = &escape( $suffix );
  s|(.*)$suffix|$1|;
  $_;
}


local( $prog ) = &basename( $0 );
local( $usage ) = "Usage: $prog [-q] [-d] infile outfile indexfile titlefile [title]";

local( $quiet ) = $FALSE;
local( $debug ) = $FALSE;
local( $dflag ) = "";

while ( $ARGV[0] =~ m|^-| )
{
  $_ = shift;
  last if m|^--|;

  if ( m|^-q$| )
  {
    $quiet = $TRUE; next;
  }
  elsif ( m|^-d$| )
  {
    $debug = $TRUE; $dflag = "-d"; next;
  }

  print "$prog: ERROR: Invalid flag: \"$_\"";
  print "$usage";
  exit( 1 );
}

if ( $#ARGV < 3 && $#ARGV > 4 )
{
  print "$prog: ERROR: Wrong number of parameters specified";
  print "$usage";
  exit( 1 );
}
  
local( $infile ) = "$ARGV[0]";
local( $outfile ) = "$ARGV[1]";		# <name>.tmp.000HT

if ( ! -e "$infile.mif" || ! -f "$infile.mif" )
  { die "$prog: $infile.mif does not exist or is not a file... exiting\n"; }

$ENV{'TRANSFilename'} = "$infile";
$ENV{'HEADFilename'} = "$infile.tmp.000H";

$ENV{'TITLEFilename'} = "$ARGV[2]";	# <name>.tmp.000T
$ENV{'INDEXFilename'} = "$ARGV[3]";	# <name>.tmp.000I
$ENV{'TITLE'} = "$ARGV[4]";

$quiet == $FALSE && print
  "\n$prog: Starting translation of MIF file $infile.mif to $outfile...";

&extractmifhead( $infile );

eval system( "-BINDIR-/trans $dflag < $infile.mif" );
eval system( "cat $infile.tmp.000F $infile.tmp.000P | -SCRDIR-/instext | -BINDIR-/insertcharacters > $outfile" );

$debug == $FALSE &&
  eval system( "rm -f $infile.tmp.000F $infile.tmp.000P $infile.tmp.000H" );
close( OUTFILE );

$quiet == $FALSE && print "\n$prog: Done\n";

exit( 0 );


# extractmifhead
# Extracts everything necessary to create a header for figure and table files.
# Adds a special paragraph format (normal text) to be used by the mif2gif
# filter.
# Input:  <miffile> with ".mif" stripped, i.e. <basename>
# Output: <basename>.tmp.000H
sub extractmifhead
{
  local( $infile ) = @_;
  local( $idline );

  open( INFILE, "<$infile.mif" ) ||
    die "$prog: Could not open input file $infile... exiting\n";
  open( OUTFILE, ">$infile.tmp.000H" ) ||
    die "$prog: Could not open temporary file $infile.tmp.000H for writing... exiting\n";

  $inside = $FALSE;

  $idline = <INFILE>;
  $idline =~ s|^([^#]*).*|$1 # Generated by Jon S. v. Tetzchner|;
  print OUTFILE "$idline";

  while ( <INFILE> )
  {
    chop;
    if ( m|<PgfCatalog| || m|<FontCatalog| || m|<RulingCatalog| ||
	 m|<TblCatalog| || m|<AFrames| || m|<Tbls| || m|<Document | )
    {
      $inside = $TRUE;
      print OUTFILE;
    }
    elsif ( m|> # end of FontCatalog| || m|> # end of RulingCatalog| ||
	    m|> # end of TblCatalog| || m|> # end of AFrames| ||
	    m|> # end of Tbls| || m|> # end of Document| )
    {
      $inside = $FALSE;
      print OUTFILE;
    }
    elsif ( m|> # end of PgfCatalog| )
    {
      $inside = $FALSE;
      $endpgf = $_;
      print OUTFILE " <Pgf";
      print OUTFILE "  <PgfTag `fm2htmlparagraphformat'>";
      print OUTFILE "  <PgfUseNextTag No >";
      print OUTFILE "  <PgfNextTag `'>";
      print OUTFILE "  <PgfAlignment LeftRight >";
      print OUTFILE "  <PgfFIndent  0.0 cm>";
      print OUTFILE "  <PgfLIndent  0.0 cm>";
      print OUTFILE "  <PgfRIndent  0.0 cm>";
      print OUTFILE "  <PgfTopSeparator `'>";
      print OUTFILE "  <PgfBotSeparator `'>";
      print OUTFILE "  <PgfPlacement Anywhere >";
      print OUTFILE "  <PgfSpBefore  4.0 pt>";
      print OUTFILE "  <PgfSpAfter  2.0 pt>";
      print OUTFILE "  <PgfWithPrev No >";
      print OUTFILE "  <PgfWithNext No >";
      print OUTFILE "  <PgfBlockSize 2>";
      print OUTFILE "  <PgfFont ";
      print OUTFILE "   <FTag `'>";
      print OUTFILE "   <FFamily `Times'>";
      print OUTFILE "   <FVar `Regular'>";
      print OUTFILE "   <FWeight `Regular'>";
      print OUTFILE "   <FAngle `Regular'>";
      print OUTFILE "   <FSize  12.0 pt>";
#	            print OUTFILE "   <FUnderline No >";
      print OUTFILE "   <FOverline No >";
      print OUTFILE "   <FStrike No >";
#		    print OUTFILE "   <FSupScript No >";
#		    print OUTFILE "   <FSubScript No >";
      print OUTFILE "   <FChangeBar No >";
      print OUTFILE "   <FOutline No >";
      print OUTFILE "   <FShadow No >";
      print OUTFILE "   <FPairKern No >";
#		    print OUTFILE "   <FDoubleUnderline No >";
#		    print OUTFILE "   <FNumericUnderline No >";
      print OUTFILE "   <FDX  0.0 pt>";
      print OUTFILE "   <FDY  0.0 pt>";
      print OUTFILE "   <FDW  0.0 pt>";
      print OUTFILE "   <FSeparation 0>";
      print OUTFILE "  > # end of PgfFont";
      print OUTFILE "  <PgfLineSpacing Proportional >";
      print OUTFILE "  <PgfLeading  2.0 pt>";
      print OUTFILE "  <PgfAutoNum No >";
      print OUTFILE "  <PgfNumTabs 0>";
      print OUTFILE "  <PgfHyphenate Yes >";
      print OUTFILE "  <HyphenMaxLines 4>";
      print OUTFILE "  <HyphenMinPrefix 2>";
      print OUTFILE "  <HyphenMinSuffix 2>";
      print OUTFILE "  <HyphenMinWord 5>";
      print OUTFILE "  <PgfLetterSpace No >";
      print OUTFILE "  <PgfMinWordSpace 75>";
      print OUTFILE "  <PgfOptWordSpace 100>";
      print OUTFILE "  <PgfMaxWordSpace 125>";
      print OUTFILE "  <PgfLanguage USEnglish >";
      print OUTFILE "  <PgfCellAlignment Top >";
      print OUTFILE "  <PgfCellMargins  0.0 pt 0.0 pt 0.0 pt 0.0 pt>";
      print OUTFILE "  <PgfCellLMarginFixed No >";
      print OUTFILE "  <PgfCellTMarginFixed No >";
      print OUTFILE "  <PgfCellRMarginFixed No >";
      print OUTFILE "  <PgfCellBMarginFixed No >";
      print OUTFILE " > # end of Pgf";
      print OUTFILE "$endpgf";
    }
    elsif ( $inside == $TRUE )
    {
      if ( m|<DTwoSides| )
      {
	print OUTFILE " <DTwoSides No >";
      }
      elsif ( m|<DPageRounding| )
      {
	print OUTFILE " <DPageRounding DeleteEmptyPages >";
      }
      else
      {	
	print OUTFILE;
      }
    }
  }

  close( INFILE );
  close( OUTFILE );
}
